<?php

namespace cs267\hw3;

use seekquarry\yioop\library\Library;
use seekquarry\yioop\library\FetchUrl;
use seekquarry\yioop\library\CrawlConstants;
use seekquarry\yioop\library\processors\HtmlProcessor;
use seekquarry\yioop\library\PhraseParser;

require_once "vendor/autoload.php";
Library::init();

function readUrls($filename) {
    $urls = file($filename);

    $stripped_urls = [];
    foreach ($urls as $url) {
        $stripped_url = str_replace(array("\n", " "),"",$url);
        array_push($stripped_urls, $stripped_url);
    }

    return $stripped_urls;
}

function getPagesInfo($urls) {
    # collect all the pages
    $pages=[];
    foreach ($urls as $url) {
        array_push($pages, [CrawlConstants::URL => $url]);
    }

    # download a collection of web pages and then pretty print
    $pages_info = FetchUrl::getPages($pages);

    return $pages_info;
}

$filename = $argv[1];
$locale = $argv[2];

$urls = readUrls($filename);
$size = count($urls);
$pages_info = getPagesInfo($urls);

$htmlprocessor = new HtmlProcessor(
    $max_description_len=20000, 
    $summarizer_option= CrawlConstants::CENTROID_WEIGHTED_SUMMARIZER
);

for ($idx = 0; $idx < $size; $idx++) {
    
    $page = $htmlprocessor->process($pages_info[$idx][CrawlConstants::PAGE], $urls[$idx]);

    $summary = PhraseParser::extractWordStringPageSummary($page);
    # removing tailing space, \t, \n, and \r
    $summary = rtrim($summary);
    

    if ($locale !== "none") {
        $page[CrawlConstants::LANG] = $locale;
        $stemmed_terms = PhraseParser::stemTerms(
            $summary,
            $page[CrawlConstants::LANG]
        );

        sort($stemmed_terms);

        foreach ($stemmed_terms as $stemmed_term) {
            print_r($urls[$idx].",".$stemmed_term."\n");
        }
    } else {
        $terms = explode(" ", $summary);

        sort($terms);

        foreach ($terms as $term) {
            print_r($urls[$idx].",".$term."\n");
        }
    }
}