from collections import defaultdict
import math

class TF_IDF:

    def __init__(self):
        self.query_results = [
            "query_result_1.txt",
            "query_result_2.txt",
            "query_result_3.txt",
            "query_result_4.txt",
            "query_result_5.txt",
        ]

        self.queries = [
            "how to make pancakes",
            "how to register to vote",
            "how to lose weight",
            "how to delete instagram account",
            "how to download youtube videos",
        ]
        self.NUM_QUERIES = 5
        # 5 from Google, 5 from Bing
        self.doc_N = 10
        self.doc_seen = defaultdict(set)
        self.doc_Nt = defaultdict(int)
        self.doc_tf = defaultdict(lambda : defaultdict(int))
        self.doc_idf = defaultdict(float)
        self.doc_tf_idf = defaultdict(lambda : defaultdict(float))

        self.query_N = 1
        self.query_Nt = defaultdict(int)
        self.query_tf = defaultdict(int)
        self.query_idf = defaultdict(float)
        self.query_tf_idf = defaultdict(float)

        self.result = []

    def get_query_terms(self, query):
        return query.split()

    def read(self, filename):
        with open(filename) as fr:
            lines = fr.readlines()
            fr.close()
        
        return lines

    def process_query(self, idx):
        # get query terms
        query_terms = self.get_query_terms(self.queries[idx])

        # calculate the query_tf
        for query_term in query_terms:
            self.query_tf[query_term] += 1
            self.query_Nt[query_term] = 1
        
        # calculate the query_idf
        for query_term in query_terms:
            self.query_idf[query_term] = math.log(self.query_N / self.query_Nt[query_term])

        # calculate the query_tf_idf
        for query_term in query_terms:
            self.query_tf_idf[query_term] = self.query_tf[query_term] * self.query_idf[query_term]

    def process_doc(self, idx):
        lines = self.read(self.query_results[idx])

        for line in lines:
            doc, term = line.strip("\n").split(",")
            self.doc_tf[doc][term] += 1
            self.doc_seen[term].add(doc)
        
        for term in self.doc_seen:
            self.doc_Nt[term] = len(self.doc_seen[term])

        for term in self.doc_seen:
            self.doc_idf[term] = math.log(self.doc_N * self.doc_Nt[term])

        for doc in self.doc_tf:
            for doc_term in self.doc_tf[doc]:
                self.doc_tf_idf[doc][doc_term] = self.doc_tf[doc][doc_term] * self.doc_idf[doc_term]
    
    def L2_norm(self, vec):
        val = 0
        for term in vec:
            val += vec[term]**2
        
        return math.sqrt(val)

    def dot_product(self, vec1, vec2):
        # always keep size of vec1 < size of vec2
        if len(vec1) > len(vec2):
            vec1, vec2 = vec2, vec1
        
        val = 0
        for term in vec1:
            if term in vec2:
                val += vec1[term] * vec2[term]
        
        return val

    def sim_score(self, doc):
        query_vec, doc_vec = self.query_tf, self.doc_tf_idf[doc]
        #print(query_vec, doc_vec)
        dot_product_val = self.dot_product(query_vec, doc_vec)
        L2_norm_val_query = self.L2_norm(query_vec)
        L2_norm_val_doc = self.L2_norm(doc_vec)

        if (dot_product_val == 0 or 
            L2_norm_val_query == 0 or
            L2_norm_val_doc == 0
        ):
            return 0

        return dot_product_val / L2_norm_val_query / L2_norm_val_doc

    def clear(self):
        # clear for document
        self.doc_seen.clear()
        self.doc_Nt.clear()
        self.doc_tf.clear()
        self.doc_idf.clear()
        self.doc_tf_idf.clear()

        # clear for query
        self.query_Nt.clear()
        self.query_tf.clear()
        self.query_idf.clear()
        self.query_tf_idf.clear()

        # clear the result
        self.result = []


    def run(self):

        for idx in range(self.NUM_QUERIES):
            self.process_query(idx)
            self.process_doc(idx)

            for doc in self.doc_tf:
                self.result.append( (doc, self.sim_score(doc)) )
            self.result = sorted(self.result, key = lambda x : x[1], reverse=True)

            counter = 0

            print(self.queries[idx])
            for url, score in self.result:
                print("{}. {},{}".format(counter + 1, url, score))
                counter += 1
            
            print("\n")
            self.clear()

if __name__ == "__main__":
    tf_idf = TF_IDF()
    tf_idf.run()
